function varargout = CalcStatistic(setsMatrix, numPattern, statisticFlag)

% varargout = CalcStatistic(setsMatrix, numPattern, statisticFlag)
% Calculates the value of the random variable z for the specified statistic
% in statisticFlag. 
% 
% See also READ.
%
% Copyright (c) 2001 by Fraunhofer-IGD
% $Revision: 1.0 $  $Date: 2001/07/12 $
%
% M.Arnold, 07/01
%
% -- INPUT  --
% setsMatrix: Struct array containing the extracted sets A and B for each
% individual pattern.
%
% numPattern: Number of different pattern to be tested. 
% 
% statisticFlag: String describing the statistic used for calculating the
% random variable z.
%
% -- OUTPUT -- 
% varargout: Vector containing the random variable z. Each entry
% corresponds to a different pattern.
%-------------

% Calculate the random variable z according to the specified test statistic.
switch statisticFlag
   case 'normalPDF'                                 
	varargout{1} = normalPDF(setsMatrix, numPattern);
   case 'quotientPDF'                      
	varargout{1} = quotientPDF(setsMatrix, numPattern);
   otherwise
	error(['Unknown statistic mode ''' statisticFlag '''.']);
  end

% ------------------------------------------------------------
function z_vec = normalPDF(setsMatrix, numPattern)

% Initialize vector for result of calculation of random variable
z_vec = [];
dim = size((setsMatrix(1).A));
len = dim(1)*dim(2);

for i = 0:numPattern - 1
  setA = reshape(setsMatrix(i + 1).A, 1, len);
  setB = reshape(setsMatrix(i + 1).B, 1, len);
  denom = sqrt( 1/len*( std( setA )^2 + std( setB )^2 ) );
  
  z_val = ( mean( setA ) - mean( setB ) )/( denom + eps );

  z_vec = [z_vec; 
		   z_val];
end

  
% ------------------------------------------------------------
function z_vec = quotientPDF(setsMatrix, numPattern)
% Initialise vector for result of calculation of random variable
z_vec = [];

for i = 0:numPattern - 1
  % Calculating the mean by normalizing to the number of elements is not
  % necessary, since a_m and b_m are used in a quotient to calculate z_val.
  a_m =  sum( sum( setsMatrix(i + 1).A ));
  b_m =  sum( sum( setsMatrix(i + 1).B ));
  
  z_val = (a_m  - b_m)/(a_m  + b_m + eps);

  z_vec = [z_vec; 
		   z_val];
end
% ------------------------------------------------------------








