
function patternMatrix = GeneratePatternMatrix(password, parameter) 

% patternMatrix = GeneratePatternMatrix(password, parameter)
% Generates a matrix with dimension Blocksize x BlocksPerFrame as specified
% in parameter struct. The elements of the matrix contains the values [0, 1,
% -1]. 
%
% See also WRITE.
%
% Copyright (c) 2001 by Fraunhofer-IGD
% $Revision: 1.0 $  $Date: 2001/07/12 $
%
% M.Arnold, 07/01
%
% -- INPUT --
% password: Secret key to generate random sequences.
%
% parameter: parameter struct containing Blocksize, BlocksPerFrame and vector
% of frequency indexes where the watermark will be embedded.  
%
% -- OUTPUT --
% patternMatrix: Blocksize x BlocksPerFrame matrix conaining the
% values [0, 1 , -1] to specify the change of the frequency coefficient.
% -----------------------

% Set parameter
frequencyIdxVector = parameter.frequencyIdxVector;

% Calculate number of coefficients from vector of frequency indices.
numCoeffs = length(frequencyIdxVector);

% Error check if margin idx values are contained in the frequencyIdxVector
if min(frequencyIdxVector) <= 1,
  error(['Lowest frequency index out of range in argument to function' ...
		 ' patternMatrix!']);
end

if max(frequencyIdxVector) > parameter.Blocksize/2,
  error(['Highest frequency index out of range in argument to function' ...
		 ' patternMatrix!']);
end

% Generate pattern vector with MEX-function mexRandomSeries
patternVector = mexRandomSeries(numCoeffs*parameter.BlocksPerFrame, ...
								password, 'BIT');

% Reshape vector to matrix with '1' and '0' entries.
tempMatrix = reshape(patternVector, numCoeffs, parameter.BlocksPerFrame);

% Set 0 -> -1 in order to specify the change in the fft coefficients.
tempMatrix( find(tempMatrix == 0) ) = -1;

% Assign random bit values to the matrix indexes according to
% frequencyIdxVector.
patternMatrix = zeros(parameter.Blocksize/2, parameter.BlocksPerFrame);
patternMatrix( frequencyIdxVector', ...
			   1:parameter.BlocksPerFrame) = tempMatrix; 

% Flip the matrix for multiplying the complex conjugate of the FFT
% spectra. 
patternMatrix = [zeros(1, parameter.BlocksPerFrame);...
				 patternMatrix(2:parameter.Blocksize/2, :); ...
				 zeros(1, parameter.BlocksPerFrame); ...
				 flipud(patternMatrix(2:parameter.Blocksize/2, :))];















