
function [result, message, z_array] = Read(password)

% [result, message] = Read(password)
% Reads the watermark bits from an audio file with a secret key specified by
% password. result = 1 if watermarking process is successful otherwise =
% 0. message contains a string reporting possible errors. 
%
% See also WRITE.
%
% Copyright (c) 2001 by Fraunhofer-IGD
% $Revision: 1.0 $  $Date: 2001/07/12 $
%
% M.Arnold, 07/01
%
% -- INPUT --
% password: Secret key used during the embedding process for the generation
% of the random sequences.
%
% -- OUTPUT --
% result: Flag scalar indicating success or failure of the embedding procedure.
%
% message: Message string reporting the result of the embedding process.
%-----------------------

global FFT_SIZE  
load('Common_Const.mat');

message = ['Reading process finished!'];

% --- Set parameter ---
parameter.WavFile              = 'UseFileDialog';
parameter.WavFileNew           = 'output.wav';
parameter.Blocksize            = FFT_SIZE;
parameter.BlocksPerFrame       = 10;
parameter.LeftBlocks           = 0;
parameter.RightBlocks          = 0;
parameter.AverageWidth         = parameter.LeftBlocks+parameter.RightBlocks+1;
parameter.PreProcessingMode    = 'None';
parameter.LeftOffsetSamples    = parameter.LeftBlocks*parameter.Blocksize;
parameter.RightOffsetSamples   = parameter.RightBlocks*parameter.Blocksize;
parameter.GeneralOffset        = 0;
parameter.NumberOfFilterCoeffs = FFT_SIZE/2; 
parameter.FilterGroupDelay     = parameter.NumberOfFilterCoeffs/2;
parameter.PAMResolution        = FFT_SIZE/2;
parameter.frequencyIdxVector   = [20:120];
parameter.StatisticMode        = 'normalPDF';
parameter.demo_size            = 1;

% Check if WavFile is specified. Otherwise popup file dailog. 
if isempty(parameter.WavFile) | strcmp(parameter.WavFile, 'UseFileDialog'),
  % Prepend the path to the home directory.
  if isunix
	fileName = '~/docs/research/own-paper/DrDobbs/';
  else
	fileName = 'U:\docs\research\own-paper\DrDobbs\';
  end

  % Get Filename and Path for Input-AudioFile  
  [fname, pname] = uigetfile( strcat(fileName, '*.wav') );
  if fname == 0 & pname == 0,
	return
  end
  parameter.WavFile = strcat(pname, fname);
end

% --- Load WAVE file and calculate algorithm parameter ---------------------
[properties Fs bits] = wavread(parameter.WavFile, 'size'); 
samples = properties(1);
numChannels = properties(2);  
if numChannels > 1,  
  message = ['Only for MONO files!'];
  result = 0;
end 

% Step size for one frame in samples
parameter.SamplesPerFrame = parameter.Blocksize*parameter.BlocksPerFrame; 

% Calculate total number of frames for the file (fix rounds towards zero).
total_frames = fix((samples - parameter.LeftOffsetSamples - ...
					parameter.RightOffsetSamples - parameter.GeneralOffset)/...
				   parameter.SamplesPerFrame);

% Need at least size of watermark frames for detection 
sizeWatermark = parameter.demo_size*8;
if total_frames <= sizeWatermark,
  message = ['Audio file to short to detect whole watermark!'];
  result = 0;
  return
end

% Generate the struct array pattern(i).Matrix for the '1'- and '0'-Bit.
% idxMatrix contains the indexes for A and B sets.
for i = 0:1
  key = strcat(password, num2str(i));
  pattern(i + 1).Matrix = GeneratePatternMatrix(key, parameter);
  idxMatrix(i + 1).A    = find(pattern(i + 1).Matrix == 1);
  idxMatrix(i + 1).B    = find(pattern(i + 1).Matrix == -1);
end

% Calculate length and number of pattern
lenIdxMatrix = length(idxMatrix(1).A);
numPattern   = length(idxMatrix);

% Initialize arrays for output and processing
bitFrameMat = zeros(parameter.Blocksize, parameter.BlocksPerFrame);
tempMatrix  = bitFrameMat(parameter.Blocksize, parameter.BlocksPerFrame);

% Main loop over the Frames. 
z_array = [];
for frameIdx = 0: total_frames - 1
  
  % Extract frame data
  Offset      = parameter.GeneralOffset + parameter.LeftOffsetSamples + 1; 
  StartSample = Offset + frameIdx*parameter.SamplesPerFrame;
  EndSample   = StartSample + parameter.SamplesPerFrame - 1;
  WavFrame    = wavread(parameter.WavFile, ...
						[(StartSample - parameter.LeftOffsetSamples) ...
					     (EndSample  + parameter.RightOffsetSamples)]); 
  WavFrameMat = reshape(WavFrame, ...
						parameter.Blocksize, parameter.BlocksPerFrame);

  % Take fft over COLUMNS
  FFTWavFrameMat = abs( fft( WavFrameMat ) );
  % Extract sets A and B and calculate column vector z_vec for actual frame.
  for i = 0:numPattern - 1
	setsMatrix(i + 1).A = [];
	setsMatrix(i + 1).B = [];
	
	setsMatrix(i + 1).A = ...
		[setsMatrix(i + 1).A; ...
		 reshape( FFTWavFrameMat(idxMatrix(i + 1).A), 1, lenIdxMatrix)];
	
	setsMatrix(i + 1).B = ...
		[setsMatrix(i + 1).B; ...
		 reshape( FFTWavFrameMat(idxMatrix(i + 1).B), 1, lenIdxMatrix)];
  end

  % Calculate the statistic with the extracted sets.
  z_vec = CalcStatistic(setsMatrix, numPattern, parameter.StatisticMode);
  
  % Append result for different pattern as column vector.
  z_array = [z_array z_vec];
  
end

% Find maximum values in each column and corresponding index in each column
[max_val, wmBits] = max(z_array);

% Substract 1 to get bit representation
wmBits = wmBits - 1;

% Format bit representation of watermark to matrix with 8 columns corresponding
% to the individual bits for different ASCII characters.
num = fix(length(wmBits)/8);
bitMatrix = reshape(wmBits(1:8*num), 8, num)'

% Report watermark reading result
result = 1;

