
function [result, message] = Write(watermark, password)

% [result, message] = Write(watermark, password)
% Embeds the string specified by watermark into an audio file. result = 1 if
% watermarking process is successful otherwise = 0. message contains a
% string reporting possible errors. The watermarking process is determined 
% by a secret key string in password.
%
% See also READ.
%
% Copyright (c) 2001 by Fraunhofer-IGD
% $Revision: 1.0 $  $Date: 2001/07/12 $
%
% M.Arnold, 07/01
%
% -- INPUT --
% watermark: Information string to be embedded.  
%
% password: Secret key used during embedding and detection process for
% generation of random sequences.
%
% -- OUTPUT --
% result: Flag scalar indicating success or failure of the embedding procedure.
%
% message: Message string reporting the result of the embedding process.
%-------------

% Load tables for global variables only one time
global FFT_SIZE MIN_POWER NOT_EXAMINED IRRELEVANT TONAL NON_TONAL 
global TH INDEX BARK ATH SPL Map CB LTq HAMMING_513 EXP_RAD_513 
% Constants used by the psycho acoustic model
load('Common_Const.mat');
load('Tables_fs_44100.mat');

% --- Set parameter ---
parameter.WavFile              = 'UseFileDialog';
parameter.WavFileNew           = 'marked_1.wav';
parameter.Blocksize            = FFT_SIZE;
parameter.BlocksPerFrame       = 10;
parameter.SamplesPerFrame      = parameter.Blocksize*parameter.BlocksPerFrame; 
parameter.LeftBlocks           = 0;
parameter.RightBlocks          = 0;
parameter.AverageWidth         = parameter.LeftBlocks+parameter.RightBlocks+1;
parameter.PreProcessingMode    = 'None';
parameter.LeftOffsetSamples    = parameter.LeftBlocks*parameter.Blocksize;
parameter.RightOffsetSamples   = parameter.RightBlocks*parameter.Blocksize;
parameter.GeneralOffset        = 0;
parameter.NumberOfFilterCoeffs = FFT_SIZE/2; 
parameter.FilterGroupDelay     = parameter.NumberOfFilterCoeffs/2;
parameter.PAMResolution        = FFT_SIZE/2;
parameter.LTMinWANoiseConstant = 0;
parameter.frequencyIdxVector   = [20:120];
parameter.StatisticMode        = 'quotientPDF';
parameter.watermark            = watermark;
parameter.demo_size            = 1;

% Check if WavFile is specified. Otherwise popup file dailog. 
if isempty(parameter.WavFile) | strcmp(parameter.WavFile, 'UseFileDialog'),
  % Prepend the path to the home directory.
  if isunix
	fileName = '~/docs/research/own-paper/DrDobbs/';
  else
	fileName = 'U:\docs\research\own-paper\DrDobbs\';
  end

  % Get Filename and Path for Input-AudioFile  
  [fname, pname] = uigetfile( strcat(fileName, '*.wav') );
  if fname == 0 & pname == 0,
	return
  end
  parameter.WavFile = strcat(pname, fname);
end

% --- Load WAVE file and calculate algorithm parameter ---
[properties Fs bits] = wavread(parameter.WavFile, 'size');
samples = properties(1);
numChannels = properties(2)  
if  numChannels > 1,  
  message = ['Only MONO files!'];
  result = 0;
end 

% Calculate total number of frames for the file (fix rounds towards zero).
total_frames = fix((samples - parameter.LeftOffsetSamples - ...
					parameter.RightOffsetSamples - parameter.GeneralOffset)/...
				   parameter.SamplesPerFrame);

% --- Convert from ASCII to bit representation
stringLen = length(watermark);
if stringLen ~= parameter.demo_size,
  message = ['Length of watermark doesn''t match with configured size!'];
  result = 0;
end 
wmBits = reshape(dec2bin(double(watermark), 8)', 1, stringLen*8);

% Exit if to few frames
if total_frames < length(wmBits),
  message = ['Audio file to short to embed whole watermark!'];
  result = 0;
  return
end

% Write the WAVE header for output file
fmt = wavwrite_Header(samples, numChannels, Fs, bits, parameter.WavFileNew);

% Copy original GeneralOffset + LeftOffsetSamples samples into output file.
EndSample = parameter.GeneralOffset + parameter.LeftOffsetSamples;
if EndSample ~= 0,
  tempFrame = wavread(parameter.WavFile, [1 EndSample]);
  [fidNew, err, samples_written] = ...
	  append_wavedat(parameter.WavFileNew, fmt, tempFrame);
  % Release memory
  clear tempFrame;
end
  
% --- Generate pattern matrix for different bits
for i = 0:1
  key = strcat(password, num2str(i));
  pattern(i + 1).Matrix = GeneratePatternMatrix(key, parameter);
end

% --- loop over bit frames --
for frameIdx = 0:total_frames - 1
  
  % Take bit frame from WAVE file
  Offset      = parameter.GeneralOffset + parameter.LeftOffsetSamples + 1;
  StartSample = Offset + frameIdx*parameter.SamplesPerFrame;
  EndSample   = StartSample + parameter.SamplesPerFrame - 1;

  WavFrame    = wavread(parameter.WavFile, [StartSample EndSample]);  
  
  % Calculate index of Bit in wmBit, which has to be embedded.
  idx = rem(frameIdx, length(wmBits)) + 1;
  pattIdx = str2num(wmBits(idx)) + 1;
  patternMatrix = pattern(pattIdx).Matrix;
  
  % Embed watermark into frame. 
  [MarkedFrame, shiftSamples] = markFrame( WavFrame(:)', ...
										   patternMatrix, parameter);
  
  % Write bit frame to output file. Don't write delay for first frame.
  if frameIdx == 0,
	[fidNew, err, samples_append] = ...
		append_wavedat(parameter.WavFileNew, fmt, ...
					   real(MarkedFrame(parameter.FilterGroupDelay+1:end,:)) );
  else
	[fidNew, err, samples_append] = ...
		append_wavedat(parameter.WavFileNew, fmt, ...
					   real(MarkedFrame) );
  end
end

% Read remaining samples
WavFrameNew = wavread(parameter.WavFile, [(EndSample + 1) samples])';

% Write delayed shiftSamples + remaining samples to file.
WavFrameNew = [shiftSamples WavFrameNew];
[fidNew, err, samples_append] = ...
	append_wavedat(parameter.WavFileNew, fmt, WavFrameNew);

% Close output file
if fclose(fidNew) == -1
  error('Error closing WAVE file!');
  result = 0;
end
% Exit and report success
result = 1;
message = ['Watermark successfully embedded!'];
return










