
function [WavFrameNew, shiftSamples] = markFrame(WavFrame, ...
												 patternMatrix, ...
												 parameter)
% [WavFrameNew, shiftSamples] = markFrame(WavFrame, patternMatrix, parameter)
% Generates the watermark noise as specified by patternMatrix and embeds it
% into one frame of audio samples of size Blocksize x BlocksPerFrame.
%
% See also WRITE.
%
% Copyright (c) 2001 by Fraunhofer-IGD
% $Revision: 1.0 $  $Date: 2001/07/12 $
%
% M.Arnold, 07/01
%
% -- INPUT --
% WavFrame: Vector of samples for one frame.
%
% patternMatrix: Blocksize x BlocksPerFrame matrix conaining the values [0, 1,
% -1] to specify the change of the frequency coefficient.
% 
% parameter: parameter struct containing Blocksize, BlocksPerFrame and vector
% of frequency indexes where the watermark will be embedded.  
%
% -- OUTPUT --
% WavFrameNew: Column vector of samples of watermarked frame.
%-----------------------

% Load tables and global constants
global FFT_SIZE MIN_POWER NOT_EXAMINED IRRELEVANT INDEX SPL TONAL BARK
global NON_TONAL TH Map CB ATH LTq 

% --- Set Parameter ---
LTMinWANoiseConstant = parameter.LTMinWANoiseConstant;
SamplesPerFrame      = parameter.SamplesPerFrame;
NumberOfFilterCoeffs = parameter.NumberOfFilterCoeffs;
FilterGroupDelay     = parameter.FilterGroupDelay;
FilterLength         = parameter.NumberOfFilterCoeffs;
frequencyIdxVector   = parameter.frequencyIdxVector;
PAMResolution        = parameter.PAMResolution;
Blocksize            = parameter.Blocksize; 
BlocksPerFrame       = parameter.BlocksPerFrame;
% ---------------------

% Delay samples 
persistent FilterDelay

% Initialize FilterDelay according to call of the function
if isempty(FilterDelay)
  FilterDelay = zeros(1, FilterGroupDelay);
end

% If WavFrame is column vector, transpose to row vector
if size(WavFrame, 2) == 1,
   WavFrame = WavFrame';
end

% Delay original and add FilterDelay samples from previous frame
DelayedWavFrame = ...
	reshape( [FilterDelay WavFrame(1:end - FilterGroupDelay)], ...
			 Blocksize, BlocksPerFrame );

% Store delay for next frame
FilterDelay  = WavFrame(end - FilterGroupDelay + 1: end);
shiftSamples = FilterDelay;
% ---------------------

% Transform WavFrame vector into matrix
WavFrame = reshape(WavFrame, Blocksize, BlocksPerFrame); 

% WATERMARK NOISE GENERATION 
WmNoiseIN  = WatermarkNoiseGenerator(WavFrame, patternMatrix, parameter);

% Loop over individual blocks in frame
for blockIdx = 1:BlocksPerFrame
  
  % CALCULATION OF MASKING THRESHOLD
  [LTMinFrame( :, blockIdx), ...
   DeltaFrame( :, blockIdx)] = PsychoAcousticModel(WavFrame(:, blockIdx)',...
												   PAMResolution); 

  % FILTERING OF WATERMARK NOISE ACCORDING TO MASKING THRESHOLD
  WmNoiseOUT(:, blockIdx) = FilterWmNoise(WmNoiseIN( :, blockIdx)', ...
										  LTMinFrame( :, blockIdx)', ...
										  parameter); 
end   

% ADJUST POWER OF WATERMARK NOISE
WmNoiseOUT = ScaleWmNoise(WmNoiseOUT, LTMinFrame, DeltaFrame, ...
						  LTMinWANoiseConstant);

% Add watermark signal to delayed original
if (sum( isnan(WmNoiseOUT)) | sum(isinf(WmNoiseOUT)) )
  WavFrameNew = DelayedWavFrame;
else 					        
  WavFrameNew = DelayedWavFrame + real(WmNoiseOUT); 
end 

% Reshaping to a column vector
WavFrameNew = reshape(WavFrameNew, SamplesPerFrame, 1);
