package xander;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.Vector;
import java.util.Iterator;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.adapters.DOMAdapter;
import org.jdom.input.DOMBuilder;

/**
 * <code>Config</code> is a utility class used to read the Xander config file.
 * The class is implemented as a Singleton. Please read the config file and the 
 * DTD for more information.
 * This class uses <a href="http://www.jdom.org">JDOM<a> beta 4 from the  for parsing the file.<p>
 * 
 * last modified - 17 Dec 2000<br>
 * version 1.0<br>
 * author Chang Sau Sheong
 *
 */

public class Config {
  
  private static Config instance;
  
  // default XML class to read the mail format from
  private String _fileName = "xander.xml";
  
  // default builder uses Apache Xerces parser
  private String _builder = "org.jdom.adapters.XercesDOMAdapter";


	// configuration for the synchronizer	
	private String _discoveryServer = new String();
	private String _discoveryPort = new String();

	private String _primaryServer = new String();
	private String _primaryPort = new String();
	
	private int _maxServerRetry = 0;
	private int _maxPrimaryRetry = 0;
	private float _waitTimer = 0;
	private int _timeout = 0;

	// configuration for the application
	private String _appName = new String();
	private String _appClassName = new String();
	
	/**
	Get the application name.
	*/
	public String getAppName() {
		return _appName;	
	}
	
	/**
	Get the application class name
	*/
	public String getAppClassName() {
		return _appClassName;
	}

	/**
	Get the maximum no of re-tries before declaring the server is un-reachable
	*/
	public int getMaxServerRetry() {
		return _maxServerRetry;
	}

	/**
	Get the maximum no of re-tries before declaring the primary server is un-reachable
	*/
	public int getMaxPrimaryRetry() {
		return _maxPrimaryRetry;
	}

	/**
	Get the wait timer, a variable used to calculate the amount of time to wait
	in between threads.
	*/

	public float getWaitTimer() {
		return _waitTimer;
	}

	/**
	Get the no of seconds to wait before timing out a multicast datagram
	*/
	public int getTimeout() {
		return (_timeout * 1000);
	}
	
	/**
	Get the multicast discovery server
	*/	
	public String getDiscoveryServer() {
		return _discoveryServer;
	}
	
	/**
	Get the multicast discovery port
	*/	
	public int getDiscoveryPort() {
		return new Integer(_discoveryPort).intValue();
	}
	
	/**
	Get the primary multicast server
	*/	
	public String getPrimaryServer() {
		return _primaryServer;
	}
	
	/**
	Get the primary multicast port
	*/	
	public int getPrimaryPort() {
		return new Integer(_primaryPort).intValue();
	}


  /**
  Creates the Config class to store the calendar server configurations.
  Private constructor for Singleton implementation.
  */
  private Config() {
    try {
    	parse();
    }
    catch (Exception e) {
    	System.out.println("Cannot create a Config object." + e.getMessage());
    }
  }
  
  /**
  Get the Singleton instance.
  */
  public static synchronized Config getInstance() {
    if (instance == null) {
      instance = new Config();
    }
    return instance;
  }
  


  /**
  Refreshes the instance with values from the default XML file.
  */
  public void refresh() {
    try {
    	parse();
    }
    catch (Exception e) {
    	System.out.println("Cannot create a Config object." + e.getMessage());
    } 	
  }


  /**
  Sets the JDOM builder adapter.
  @param classname The JDOM builder adapter class name.
  */
  public void setBuilder(String classname) {
    _builder = classname;
  }

  /**
  Gets the JDOM builder adapter.
  */
  public String getBuilder() {
    return _builder;
  }


  /**
  Parses the XML config file.
  */
  private void parse() throws IOException {
    try {

			// Request DOM Implementation and Xerces Parser
			DOMBuilder builder = new DOMBuilder(_builder);

       //build a DOM tree with the specified or default parser
 			DOMAdapter domAdapter;

  		Class parserClass = Class.forName(_builder);
			domAdapter = (DOMAdapter)parserClass.newInstance();

      org.w3c.dom.Document w3cdoc = domAdapter.getDocument(getClass().getResourceAsStream(_fileName), false);

      Document doc = builder.build(w3cdoc);

      // Get the root element
      Element root = doc.getRootElement();
        
      // Get the various elements
      
      // Get the multicast discovery server
      _discoveryServer = root.getChild("synchronizer").getChild("discovery").getChild("server").getContent();
 
      // Get the multicast discovery port
      _discoveryPort = root.getChild("synchronizer").getChild("discovery").getChild("port").getContent();

      // Get the primary multicast server
      _primaryServer = root.getChild("synchronizer").getChild("primary").getChild("server").getContent();
 
      // Get the primary multicast port
      _primaryPort = root.getChild("synchronizer").getChild("primary").getChild("port").getContent();

			// Get the number of retries before confirming that the server is down and remove it from the current cluster status
			_maxServerRetry = Integer.parseInt(root.getChild("synchronizer").getChild("max_server_retry").getContent());

			// Get the number of retries before confirming that the primary server is down
			// Begin primary contention
			_maxPrimaryRetry = Integer.parseInt(root.getChild("synchronizer").getChild("max_primary_retry").getContent());

			// Get the wait timer variable that controls the max amount of time in between threads
			_waitTimer = Float.parseFloat(root.getChild("synchronizer").getChild("wait_timer").getContent());

			// Get the timeout variable that controls the max amount of time to wait for the multicast datagrams before
			_timeout = Integer.parseInt(root.getChild("synchronizer").getChild("timeout").getContent()); 
			
			// Get the application name and class name to instantiate
			_appName = root.getChild("app").getChild("name").getContent();
			_appClassName = root.getChild("app").getChild("class_name").getContent();
      
    }

    catch (JDOMException e) {
      throw new IOException("Cannot parse document." + e.getMessage());
    }

    catch (Exception e) {
    	System.out.println("Cannot parse document." + e.getMessage());
			e.printStackTrace();
    }
  }
}